<?php

namespace Drupal\commerce_shipping_colissimo;

use Drupal\commerce_checkout\Plugin\Commerce\CheckoutFlow\CheckoutFlowInterface;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_shipping\Entity\ShipmentInterface;
use Drupal\commerce_shipping\Entity\ShippingMethod;
use Drupal\commerce_shipping\Entity\ShippingMethodInterface;
use Drupal\commerce_shipping\Form\ShipmentForm;
use Drupal\commerce_shipping_colissimo\Plugin\Commerce\ShippingMethod\Colissimo;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\EntityReferenceFieldItemListInterface;
use Drupal\Core\Form\BaseFormIdInterface;
use Drupal\Core\Form\FormStateInterface;

class RelayFormHelper {
  private EntityTypeManagerInterface $entityTypeManager;

  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  public function isColissimoRelay(FormStateInterface $formState): bool {
    $shippingMethod = $this->extractSelectedShippingMethod($formState);
    if (!($shippingMethod instanceof ShippingMethod)) {
      return FALSE;
    }
    $plugin = $shippingMethod->getPlugin();
    if (!($plugin instanceof Colissimo)) {
      return FALSE;
    }
    return $plugin->isRelay();
  }

  protected function extractSelectedShippingMethod(FormStateInterface $formState): ?ShippingMethodInterface {
    $shippingMethodKey = @$formState->getUserInput()['shipping_information']['shipments'][0]['shipping_method'][0]
                    ?: @$formState->getUserInput()['shipping_method'][0];
    if ($shippingMethodKey) {
      $shippingMethodId = (int) filter_var($shippingMethodKey, FILTER_SANITIZE_NUMBER_INT);
      $shippingMethod = $this->entityTypeManager->getStorage('commerce_shipping_method')->load($shippingMethodId);
      assert($shippingMethod instanceof ShippingMethodInterface);
      return $shippingMethod;
    }
    $callback = @$formState->getBuildInfo()['callback_object'];
    if ($callback instanceof ShipmentForm) {
      $shipment = $callback->getEntity();
      assert($shipment instanceof ShipmentInterface);
      return $shipment->getShippingMethod();
    }

    $order = $this->extractOrder($formState);
    $shipmentsField = $order->get('shipments');
    assert($shipmentsField instanceof EntityReferenceFieldItemListInterface);
    $shipments = $shipmentsField->referencedEntities();
    $shipment = reset($shipments);
    if (!($shipment instanceof ShipmentInterface)) {
      return NULL;
    }
    return $shipment->getShippingMethod();
  }

  private function extractOrder(FormStateInterface $formState): ?OrderInterface {
    $formObject = $formState->getFormObject();
    if (!($formObject instanceof BaseFormIdInterface)) {
      return NULL;
    }

    if ($formObject instanceof EntityFormInterface) {
      $entity = $formObject->getEntity();
      if ($entity instanceof OrderInterface) {
        return $entity;
      }
    }
    if ($formObject instanceof CheckoutFlowInterface) {
      return $formObject->getOrder();
    }

    return NULL;
  }

}
