<?php

namespace Drupal\commerce_shipping_colissimo;

class Setting {
  private string $baseUrl;
  private ?string $user = NULL;
  private ?string $password = NULL;
  private bool $debugMode;
  private int $averagePreparationDelayInDays;
  private float $defaultParcelWeigthInKg;
  private string $labelFormat;
  private string $labelSize;
  private ?string $customerProfilePhoneField;
  private string $labelSenderParcelIdSource;

  public function isConfigured(): bool {
    return $this->user && $this->password;
  }

  public function getBaseUrl(): string {
    return $this->baseUrl;
  }

  public function setBaseUrl(string $baseUrl): void {
    $this->baseUrl = $baseUrl;
  }

  public function getUser(): ?string {
    return $this->user;
  }

  public function setUser(?string $user): void {
    $this->user = $user;
  }

  public function getPassword(): ?string {
    return $this->password;
  }

  public function setPassword(?string $password): void {
    $this->password = $password;
  }

  public function isDebugMode(): bool {
    return $this->debugMode;
  }

  public function setDebugMode(bool $debugMode): void {
    $this->debugMode = $debugMode;
  }

  public function getAveragePreparationDelayInDays(): int {
    return $this->averagePreparationDelayInDays;
  }

  public function setAveragePreparationDelayInDays(int $averagePreparationDelayInDays): void {
    if ($averagePreparationDelayInDays < 0) {
      throw new \UnexpectedValueException('Preparation delay cannot be negative');
    }
    $this->averagePreparationDelayInDays = $averagePreparationDelayInDays;
  }

  public function getDefaultParcelWeigthInKg(): float {
    return $this->defaultParcelWeigthInKg;
  }

  public function setDefaultParcelWeigthInKg(float $defaultParcelWeigthInKg): void {
    if ($defaultParcelWeigthInKg < 0) {
      throw new \UnexpectedValueException('Parcel weight cannot be negative');
    }
    $this->defaultParcelWeigthInKg = $defaultParcelWeigthInKg;
  }

  public function getLabelFormat(): string {
    return $this->labelFormat;
  }

  public function setLabelFormat(string $labelFormat): void {
    LabelFormat::assertExists($labelFormat);
    $this->labelFormat = $labelFormat;
  }

  public function getLabelSize(): string {
    return $this->labelSize;
  }

  public function setLabelSize(string $labelSize): void {
    LabelSize::assertExists($labelSize);
    $this->labelSize = $labelSize;
  }

  public function getCustomerProfilePhoneField(): ?string {
    return $this->customerProfilePhoneField;
  }

  public function setCustomerProfilePhoneField(?string $customerProfilePhoneField): void {
    $this->customerProfilePhoneField = $customerProfilePhoneField;
  }

  public function getLabelSenderParcelIdSource(): string {
    return $this->labelSenderParcelIdSource;
  }

  public function setLabelSenderParcelIdSource(string $labelSenderParcelIdSource): void {
    LabelSenderParcelIdSource::assertExists($labelSenderParcelIdSource);
    $this->labelSenderParcelIdSource = $labelSenderParcelIdSource;
  }

}
